@@ -26,7 +26,15 @@ from crispy_forms.tests.utils import contains_partial
 from crispy_forms.utils import render_crispy_form
 
 from .conftest import only_bootstrap, only_bootstrap4
-from .forms import CheckboxesSampleForm, SampleForm
+from .forms import (
+    CheckboxesSampleForm,
+    CustomCheckboxSelectMultiple,
+    CustomRadioSelect,
+    GroupedChoiceForm,
+    SampleForm,
+    SampleFormCustomWidgets,
+)
+from .utils import parse_expected, parse_form
 
 
 def test_field_with_custom_template():
@@ -154,15 +162,10 @@ def test_remove_labels():
 @only_bootstrap
 class TestBootstrapLayoutObjects:
     def test_custom_django_widget(self, settings):
-        class CustomRadioSelect(forms.RadioSelect):
-            pass
-
-        class CustomCheckboxSelectMultiple(forms.CheckboxSelectMultiple):
-            pass
 
         # Make sure an inherited RadioSelect gets rendered as it
-        form = CheckboxesSampleForm()
-        form.fields["inline_radios"].widget = CustomRadioSelect()
+        form = SampleFormCustomWidgets()
+        assert isinstance(form.fields["inline_radios"].widget, CustomRadioSelect)
         form.helper = FormHelper()
         form.helper.layout = Layout("inline_radios")
 
@@ -173,7 +176,7 @@ class TestBootstrapLayoutObjects:
             assert 'class="radio"' in html
 
         # Make sure an inherited CheckboxSelectMultiple gets rendered as it
-        form.fields["checkboxes"].widget = CustomCheckboxSelectMultiple()
+        assert isinstance(form.fields["checkboxes"].widget, CustomCheckboxSelectMultiple)
         form.helper.layout = Layout("checkboxes")
         html = render_crispy_form(form)
         if settings.CRISPY_TEMPLATE_PACK == "bootstrap4":
@@ -241,6 +244,19 @@ class TestBootstrapLayoutObjects:
         assert html.count("custom-select") == 1
         assert html.count("USD") == 1
 
+    @only_bootstrap4
+    def test_prepended_appended_text_input_size(self, settings):
+        test_form = SampleForm()
+        test_form.helper = FormHelper()
+        test_form.helper.layout = Layout(
+            PrependedAppendedText("email", "@", "gmail.com", input_size="input-group-lg"),
+            AppendedText("password1", "#", input_size="input-group-sm"),
+            PrependedText("password2", "$", input_size="input-group-lg"),
+        )
+        html = render_crispy_form(test_form)
+        assert html.count('<div class="input-group input-group-lg">') == 2
+        assert html.count('<div class="input-group input-group-sm">') == 1
+
     def test_inline_radios(self, settings):
         test_form = CheckboxesSampleForm()
         test_form.helper = FormHelper()
@@ -487,7 +503,7 @@ class TestBootstrapLayoutObjects:
         test_form = CheckboxesSampleForm()
         html = render_crispy_form(test_form)
 
-        assert html.count('checked="checked"') == 6
+        assert html.count("checked") == 6
 
         test_form.helper = FormHelper(test_form)
         test_form.helper[1].wrap(InlineCheckboxes, inline=True)
@@ -508,16 +524,34 @@ class TestBootstrapLayoutObjects:
         html = render_crispy_form(test_form)
 
         expected_ids = [
+            "checkboxes_0",
             "checkboxes_1",
             "checkboxes_2",
-            "checkboxes_3",
+            "alphacheckboxes_0",
             "alphacheckboxes_1",
             "alphacheckboxes_2",
-            "alphacheckboxes_3",
+            "numeric_multiple_checkboxes_0",
             "numeric_multiple_checkboxes_1",
             "numeric_multiple_checkboxes_2",
-            "numeric_multiple_checkboxes_3",
         ]
         for id_suffix in expected_ids:
-            expected_str = 'id="id_{id_suffix}"'.format(id_suffix=id_suffix)
+            expected_str = f'id="id_{id_suffix}"'
             assert html.count(expected_str) == 1
+
+    @only_bootstrap4
+    def test_grouped_checkboxes_radios(self):
+        form = GroupedChoiceForm()
+        form.helper = FormHelper()
+        form.helper.layout = Layout("checkbox_select_multiple")
+        assert parse_form(form) == parse_expected("bootstrap4/test_layout_objects/test_grouped_checkboxes.html")
+        form.helper.layout = Layout("radio")
+        assert parse_form(form) == parse_expected("bootstrap4/test_layout_objects/test_grouped_radios.html")
+
+        form = GroupedChoiceForm({})
+        form.helper = FormHelper()
+        form.helper.layout = Layout("checkbox_select_multiple")
+        assert parse_form(form) == parse_expected(
+            "bootstrap4/test_layout_objects/test_grouped_checkboxes_failing.html"
+        )
+        form.helper.layout = Layout("radio")
+        assert parse_form(form) == parse_expected("bootstrap4/test_layout_objects/test_grouped_radios_failing.html")
